/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
import { ChangeDetectorRef } from '@angular/core';
import { Optional } from '@angular/core';
import { ConnectService } from '../services/connect.service';
import { Directive, EventEmitter, Input, Output } from '@angular/core';
import { NqConnectedComponent } from './connected.component';
export class ConnectRequestDirective {
    /**
     * @param {?} connectService
     * @param {?} changeDetector
     * @param {?} host
     */
    constructor(connectService, changeDetector, host) {
        this.connectService = connectService;
        this.changeDetector = changeDetector;
        this.host = host;
        // tslint:disable-next-line:no-input-rename
        this.config = /** @type {?} */ (undefined);
        this.response = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.subscribe(this.config);
        if (this.host) {
            this.host.nqRefresh.subscribe(() => this.forceRequest());
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.unsubscribe();
    }
    /**
     * @param {?} config
     * @return {?}
     */
    subscribe(config) {
        this.unsubscribe();
        this.subscription = this.connectService.requestAsync(config).subscribe(response => {
            if (this.response) {
                if (this.host) {
                    setTimeout(() => {
                        this.host.nqData = response;
                        if (this.host.changeDetector) {
                            this.host.changeDetector.markForCheck();
                        }
                    });
                }
                this.response.emit(response);
            }
        });
    }
    /**
     * @return {?}
     */
    unsubscribe() {
        if (this.subscription) {
            this.subscription.unsubscribe();
        }
    }
    /**
     * @return {?}
     */
    forceRequest() {
        this.subscribe(Object.assign({}, this.config, { force: true }));
    }
}
ConnectRequestDirective.decorators = [
    { type: Directive, args: [{
                exportAs: 'nqConnect',
                selector: '[nqConnect]',
            },] },
];
/** @nocollapse */
ConnectRequestDirective.ctorParameters = () => [
    { type: ConnectService, },
    { type: ChangeDetectorRef, },
    { type: NqConnectedComponent, decorators: [{ type: Optional },] },
];
ConnectRequestDirective.propDecorators = {
    "config": [{ type: Input, args: ['nqConnect',] },],
    "response": [{ type: Output },],
};
function ConnectRequestDirective_tsickle_Closure_declarations() {
    /** @type {!Array<{type: !Function, args: (undefined|!Array<?>)}>} */
    ConnectRequestDirective.decorators;
    /**
     * @nocollapse
     * @type {function(): !Array<(null|{type: ?, decorators: (undefined|!Array<{type: !Function, args: (undefined|!Array<?>)}>)})>}
     */
    ConnectRequestDirective.ctorParameters;
    /** @type {!Object<string,!Array<{type: !Function, args: (undefined|!Array<?>)}>>} */
    ConnectRequestDirective.propDecorators;
    /** @type {?} */
    ConnectRequestDirective.prototype.config;
    /** @type {?} */
    ConnectRequestDirective.prototype.response;
    /** @type {?} */
    ConnectRequestDirective.prototype.subscription;
    /** @type {?} */
    ConnectRequestDirective.prototype.connectService;
    /** @type {?} */
    ConnectRequestDirective.prototype.changeDetector;
    /** @type {?} */
    ConnectRequestDirective.prototype.host;
}
//# sourceMappingURL=connect.directive.js.map