/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
import * as tslib_1 from "tslib";
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Inject, Injectable } from '@angular/core';
import { Actions, Effect } from '@ngrx/effects';
import { Store } from '@ngrx/store';
import * as actions from 'redux-query/dist/es/actions';
import * as actionTypes from 'redux-query/dist/es/constants/action-types';
import * as httpMethods from 'redux-query/dist/es/constants/http-methods';
import { getQueryKey } from 'redux-query/dist/es/lib/query-key';
import { Observable, of, range, timer } from 'rxjs';
import { catchError, filter, flatMap, map, mergeMap, retryWhen, tap, zip } from 'rxjs/operators';
import * as ngrxQueryActionTypes from '../helpers/actionTypes';
import { NGRX_QUERY_CONFIG } from '../helpers/config';
import { getLatest } from '../helpers/getLatest';
import { invariant } from '../helpers/invariant';
import { MockMode } from '../helpers/mockMode';
import { ofType } from '../helpers/ofType';
import { defaultBackoffConfig, defaultEntitiesSelector, defaultQueriesSelector, defaultRetryableStatusCodes } from './../helpers/config';
/**
 * @param {?} x
 * @param {?=} y
 * @param {?=} z
 * @return {?}
 */
export function identity(x, y, z) {
    return x;
}
/**
 * @param {?} optimisticUpdate
 * @param {?} entities
 * @return {?}
 */
export function optimisticUpdateEntities(optimisticUpdate, entities) {
    return Object.keys(optimisticUpdate).reduce((accum, key) => {
        if (optimisticUpdate[key]) {
            accum[key] = optimisticUpdate[key](entities[key]);
        }
        else {
            accum[key] = entities[key];
        }
        return accum;
    }, {});
}
;
/**
 * @param {?} update
 * @param {?} entities
 * @param {?} transformed
 * @return {?}
 */
export function updateEntities(update, entities, transformed) {
    // If update not supplied, then no change to entities should be made
    return Object.keys(update || {}).reduce((accum, key) => {
        accum[key] = update[key]((entities || {})[key], (transformed || {})[key]);
        return accum;
    }, {});
}
;
/**
 * @param {?} queries
 * @return {?}
 */
export function getPendingQueries(queries) {
    return queries.filter((query) => query.isPending);
}
;
export class NgrxQueryEffects {
    /**
     * @param {?} actions$
     * @param {?} store
     * @param {?} http
     * @param {?} config
     */
    constructor(actions$, store, http, config) {
        this.actions$ = actions$;
        this.store = store;
        this.http = http;
        this.config = config;
        this.requestAsync = this.actions$.pipe(ofType(ngrxQueryActionTypes.REQUEST_ASYNC), map((action) => {
            const { url, force, retry, update, } = action;
            invariant(!!url, 'Missing required `url` field in action handler');
            invariant(!!update, 'Missing required `update` field in action handler');
            const /** @type {?} */ queryKey = getQueryKey(action);
            const /** @type {?} */ state = getLatest(this.store);
            const /** @type {?} */ queries = this.config && this.config.queriesSelector && this.config.queriesSelector(state)
                || defaultQueriesSelector(state);
            const /** @type {?} */ queriesState = queries[queryKey];
            const { isPending, status } = queriesState || /** @type {?} */ ({});
            const /** @type {?} */ hasSucceeded = status >= 200 && status < 300;
            const /** @type {?} */ attemptRequest = force || !queriesState || (retry !== false && !isPending && !hasSucceeded);
            return { attemptRequest, action };
        }), filter(({ attemptRequest }) => attemptRequest), mergeMap(({ action }) => {
            const { url, body, transform = identity, update, options = /** @type {?} */ ({}), meta, } = action;
            const /** @type {?} */ queryKey = getQueryKey(action);
            const /** @type {?} */ start = new Date();
            const { method = httpMethods.GET } = options;
            const /** @type {?} */ request = {
                body,
                headers: options.headers,
                method,
                observe: /** @type {?} */ ((options.observe || 'response')),
                responseType: options.responseType,
                url,
                withCredentials: options.credentials === 'include',
            };
            return of({}).pipe(tap(() => {
                this.store.dispatch(actions.requestStart({ url, body, request, meta, queryKey }));
            }), filter(() => {
                if (this.config && this.config.mock && this.config.mock.mode === MockMode.Mock) {
                    const /** @type {?} */ mockActions = this.config.mock.getMockData(queryKey);
                    if (mockActions) {
                        mockActions.forEach(mockAction => {
                            this.store.dispatch(mockAction);
                        });
                    }
                    else {
                        this.store.dispatch(actions.requestFailure({
                            url,
                            body,
                            status: 500,
                            responseBody: 'No mock data recorded for ' + queryKey,
                            meta,
                            queryKey,
                        }));
                    }
                    return false;
                }
                return true;
            }), mergeMap(() => {
                return this.http.request(request.method, url, request)
                    .pipe(map(response => {
                    if (!response.ok) {
                        throw response;
                    }
                    return response;
                }));
            }), map(response => {
                const /** @type {?} */ callbackState = getLatest(this.store);
                const /** @type {?} */ entities = this.config && this.config.entitiesSelector && this.config.entitiesSelector(callbackState)
                    || defaultEntitiesSelector(callbackState);
                const /** @type {?} */ parsedResponse = response.body;
                const /** @type {?} */ transformed = transform(parsedResponse, response.body, response);
                const /** @type {?} */ newEntities = updateEntities(update, entities, transformed);
                const /** @type {?} */ requestSuccessAction = actions.requestSuccess({
                    url, body, status: response.status, entities: newEntities, meta, queryKey, responseBody: parsedResponse,
                    responseText: response.body, responseHeaders: response.headers
                });
                this.store.dispatch(requestSuccessAction);
                const /** @type {?} */ end = new Date();
                const /** @type {?} */ duration = end.valueOf() - start.valueOf();
                const /** @type {?} */ requestAsyncAction = {
                    body: parsedResponse,
                    duration,
                    entities: newEntities,
                    meta,
                    status: response.status,
                    text: response.body,
                    transformed,
                    type: actionTypes.REQUEST_ASYNC,
                };
                if (this.config && this.config.mock && this.config.mock.mode === MockMode.Record) {
                    this.config.mock.saveMockData(queryKey, [requestSuccessAction, requestAsyncAction]);
                }
                return requestAsyncAction;
            }), retryWhen(attempts => {
                const /** @type {?} */ backoff = this.config && this.config.backoff || defaultBackoffConfig;
                const /** @type {?} */ retryableStatusCodes = this.config && this.config.retryableStatusCodes || defaultRetryableStatusCodes;
                return range(1, backoff.maxAttempts + 1).pipe(zip(attempts, (i, response) => ({ response, i })), flatMap(({ response, i }) => {
                    if (i < backoff.maxAttempts && retryableStatusCodes.indexOf(response.status) >= 0) {
                        // Delay retry by min duration the first attempt, up to max duration on the (maxAttempts)th attempt
                        return timer(backoff.minDuration +
                            (i - 1) * (backoff.maxDuration - backoff.minDuration) / backoff.maxAttempts);
                    }
                    else {
                        // Tried maxAttempts, now fail
                        throw response;
                    }
                }));
            }), catchError((errResponse, caught) => {
                if (errResponse.error instanceof Error) {
                    throw errResponse.error;
                }
                const /** @type {?} */ requestFailureAction = actions.requestFailure({
                    url,
                    body,
                    status: errResponse.status,
                    responseBody: errResponse.error,
                    meta,
                    queryKey,
                    responseText: errResponse.error,
                    responseHeaders: errResponse.headers
                });
                if (this.config && this.config.mock && this.config.mock.mode === MockMode.Record) {
                    this.config.mock.saveMockData(queryKey, [requestFailureAction]);
                }
                return of(requestFailureAction);
            }));
        }));
        this.mutateAsync = this.actions$.pipe(ofType(ngrxQueryActionTypes.MUTATE_ASYNC), map((action) => {
            const { url, transform = identity, update, body, meta, optimisticUpdate, options = /** @type {?} */ ({}), } = action;
            invariant(!!url, 'Missing required `url` field in action handler');
            const /** @type {?} */ state = getLatest(this.store);
            const /** @type {?} */ entities = this.config && this.config.entitiesSelector && this.config.entitiesSelector(state)
                || defaultEntitiesSelector(state);
            let /** @type {?} */ optimisticEntities;
            if (optimisticUpdate) {
                optimisticEntities = optimisticUpdateEntities(optimisticUpdate, entities);
            }
            const /** @type {?} */ queryKey = getQueryKey(action);
            // const start = new Date();
            const { method = httpMethods.POST } = options;
            const /** @type {?} */ request = {
                body,
                headers: new HttpHeaders(options.headers),
                method,
                observe: /** @type {?} */ ((options.observe || 'response')),
                responseType: options.responseType,
                url,
                withCredentials: options.credentials === 'include',
            };
            // Note: only the entities that are included in `optimisticUpdate` will be passed along in the
            // `mutateStart` action as `optimisticEntities`
            this.store.dispatch(actions.mutateStart({ url, body, request, optimisticEntities, queryKey, meta }));
            return Object.assign({}, action, { request, entities, queryKey });
        }), filter((action) => {
            if (!this.config || !this.config.mock || this.config.mock.mode !== MockMode.Mock) {
                // Not mocking so continue
                return true;
            }
            const { url, body, meta, queryKey, } = action;
            const /** @type {?} */ mockActions = this.config.mock.getMockData(queryKey);
            if (mockActions) {
                mockActions.forEach(mockAction => {
                    this.store.dispatch(mockAction);
                });
            }
            else {
                this.store.dispatch(actions.requestFailure({
                    url,
                    body,
                    status: 500,
                    responseBody: 'No mock data recorded for ' + queryKey,
                    meta,
                    queryKey,
                }));
            }
            // Dispatched mock actions so abort network call
            return false;
        }), mergeMap((action) => {
            const { url, transform = identity, update, body, meta, request, entities, queryKey, } = action;
            return this.http.request(request.method, url, request).pipe(map(response => {
                if (!response.ok) {
                    throw response;
                }
                const /** @type {?} */ status = (response && response.status) || 0;
                const /** @type {?} */ responseBody = (response && response.body) || undefined;
                const /** @type {?} */ responseText = (response && response.body) || undefined;
                const /** @type {?} */ transformed = transform(responseBody, responseText, response);
                const /** @type {?} */ newEntities = updateEntities(update, entities, transformed);
                return actions.mutateSuccess({
                    url, body, status, entities: newEntities, queryKey, responseBody, responseText, responseHeaders: response.headers, meta
                });
            }), catchError((errResponse) => {
                if (errResponse.error instanceof Error) {
                    throw errResponse.error;
                }
                return of(actions.mutateFailure({
                    url, body, status: errResponse.status, rolledBackEntities: entities, queryKey,
                    responseBody: errResponse.error, responseText: errResponse.error, responseHeaders: errResponse.headers, meta
                }));
            }));
        }));
    }
}
NgrxQueryEffects.decorators = [
    { type: Injectable },
];
/** @nocollapse */
NgrxQueryEffects.ctorParameters = () => [
    { type: Actions, },
    { type: Store, },
    { type: HttpClient, },
    { type: undefined, decorators: [{ type: Inject, args: [NGRX_QUERY_CONFIG,] },] },
];
tslib_1.__decorate([
    Effect(),
    tslib_1.__metadata("design:type", Observable)
], NgrxQueryEffects.prototype, "requestAsync", void 0);
tslib_1.__decorate([
    Effect(),
    tslib_1.__metadata("design:type", Observable)
], NgrxQueryEffects.prototype, "mutateAsync", void 0);
function NgrxQueryEffects_tsickle_Closure_declarations() {
    /** @type {!Array<{type: !Function, args: (undefined|!Array<?>)}>} */
    NgrxQueryEffects.decorators;
    /**
     * @nocollapse
     * @type {function(): !Array<(null|{type: ?, decorators: (undefined|!Array<{type: !Function, args: (undefined|!Array<?>)}>)})>}
     */
    NgrxQueryEffects.ctorParameters;
    /** @type {?} */
    NgrxQueryEffects.prototype.requestAsync;
    /** @type {?} */
    NgrxQueryEffects.prototype.mutateAsync;
    /** @type {?} */
    NgrxQueryEffects.prototype.actions$;
    /** @type {?} */
    NgrxQueryEffects.prototype.store;
    /** @type {?} */
    NgrxQueryEffects.prototype.http;
    /** @type {?} */
    NgrxQueryEffects.prototype.config;
}
//# sourceMappingURL=effects.service.js.map
