/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
import * as tslib_1 from "tslib";
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Inject, Injectable } from '@angular/core';
import { Actions, Effect } from '@ngrx/effects';
import { Store } from '@ngrx/store';
import * as actions from 'redux-query/dist/es/actions';
import * as actionTypes from 'redux-query/dist/es/constants/action-types';
import * as httpMethods from 'redux-query/dist/es/constants/http-methods';
import { getQueryKey } from 'redux-query/dist/es/lib/query-key';
import { Observable, of, range, timer } from 'rxjs';
import { catchError, filter, flatMap, map, mergeMap, retryWhen, tap, zip } from 'rxjs/operators';
import * as ngrxQueryActionTypes from '../helpers/actionTypes';
import { NGRX_QUERY_CONFIG } from '../helpers/config';
import { getLatest } from '../helpers/getLatest';
import { invariant } from '../helpers/invariant';
import { MockMode } from '../helpers/mockMode';
import { ofType } from '../helpers/ofType';
import { defaultBackoffConfig, defaultEntitiesSelector, defaultQueriesSelector, defaultRetryableStatusCodes } from './../helpers/config';
/**
 * @param {?} x
 * @param {?=} y
 * @param {?=} z
 * @return {?}
 */
export function identity(x, y, z) {
    return x;
}
/**
 * @param {?} optimisticUpdate
 * @param {?} entities
 * @return {?}
 */
export function optimisticUpdateEntities(optimisticUpdate, entities) {
    return Object.keys(optimisticUpdate).reduce(function (accum, key) {
        if (optimisticUpdate[key]) {
            accum[key] = optimisticUpdate[key](entities[key]);
        }
        else {
            accum[key] = entities[key];
        }
        return accum;
    }, {});
}
;
/**
 * @param {?} update
 * @param {?} entities
 * @param {?} transformed
 * @return {?}
 */
export function updateEntities(update, entities, transformed) {
    // If update not supplied, then no change to entities should be made
    return Object.keys(update || {}).reduce(function (accum, key) {
        accum[key] = update[key]((entities || {})[key], (transformed || {})[key]);
        return accum;
    }, {});
}
;
/**
 * @param {?} queries
 * @return {?}
 */
export function getPendingQueries(queries) {
    return queries.filter(function (query) { return query.isPending; });
}
;
var NgrxQueryEffects = /** @class */ (function () {
    /**
     * @param {?} actions$
     * @param {?} store
     * @param {?} http
     * @param {?} config
     */
    function NgrxQueryEffects(actions$, store, http, config) {
        var _this = this;
        this.actions$ = actions$;
        this.store = store;
        this.http = http;
        this.config = config;
        this.requestAsync = this.actions$.pipe(ofType(ngrxQueryActionTypes.REQUEST_ASYNC), map(function (action) {
            var url = action.url, force = action.force, retry = action.retry, update = action.update;
            invariant(!!url, 'Missing required `url` field in action handler');
            invariant(!!update, 'Missing required `update` field in action handler');
            var /** @type {?} */ queryKey = getQueryKey(action);
            var /** @type {?} */ state = getLatest(_this.store);
            var /** @type {?} */ queries = _this.config && _this.config.queriesSelector && _this.config.queriesSelector(state)
                || defaultQueriesSelector(state);
            var /** @type {?} */ queriesState = queries[queryKey];
            var _a = queriesState || /** @type {?} */ ({}), isPending = _a.isPending, status = _a.status;
            var /** @type {?} */ hasSucceeded = status >= 200 && status < 300;
            var /** @type {?} */ attemptRequest = force || !queriesState || (retry !== false && !isPending && !hasSucceeded);
            return { attemptRequest: attemptRequest, action: action };
        }), filter(function (_a) {
            var attemptRequest = _a.attemptRequest;
            return attemptRequest;
        }), mergeMap(function (_a) {
            var action = _a.action;
            var url = action.url, body = action.body, _b = action.transform, transform = _b === void 0 ? identity : _b, update = action.update, _c = action.options, options = _c === void 0 ? ({}) : _c, meta = action.meta;
            var /** @type {?} */ queryKey = getQueryKey(action);
            var /** @type {?} */ start = new Date();
            var _d = options.method, method = _d === void 0 ? httpMethods.GET : _d;
            var /** @type {?} */ request = {
                body: body,
                headers: options.headers,
                method: method,
                observe: /** @type {?} */ ((options.observe || 'response')),
                responseType: options.responseType,
                url: url,
                withCredentials: options.credentials === 'include',
            };
            return of({}).pipe(tap(function () {
                _this.store.dispatch(actions.requestStart({ url: url, body: body, request: request, meta: meta, queryKey: queryKey }));
            }), filter(function () {
                if (_this.config && _this.config.mock && _this.config.mock.mode === MockMode.Mock) {
                    var /** @type {?} */ mockActions = _this.config.mock.getMockData(queryKey);
                    if (mockActions) {
                        mockActions.forEach(function (mockAction) {
                            _this.store.dispatch(mockAction);
                        });
                    }
                    else {
                        _this.store.dispatch(actions.requestFailure({
                            url: url,
                            body: body,
                            status: 500,
                            responseBody: 'No mock data recorded for ' + queryKey,
                            meta: meta,
                            queryKey: queryKey,
                        }));
                    }
                    return false;
                }
                return true;
            }), mergeMap(function () {
                return _this.http.request(request.method, url, request)
                    .pipe(map(function (response) {
                    if (!response.ok) {
                        throw response;
                    }
                    return response;
                }));
            }), map(function (response) {
                var /** @type {?} */ callbackState = getLatest(_this.store);
                var /** @type {?} */ entities = _this.config && _this.config.entitiesSelector && _this.config.entitiesSelector(callbackState)
                    || defaultEntitiesSelector(callbackState);
                var /** @type {?} */ parsedResponse = response.body;
                var /** @type {?} */ transformed = transform(parsedResponse, response.body, response);
                var /** @type {?} */ newEntities = updateEntities(update, entities, transformed);
                var /** @type {?} */ requestSuccessAction = actions.requestSuccess({
                    url: url, body: body, status: response.status, entities: newEntities, meta: meta, queryKey: queryKey, responseBody: parsedResponse,
                    responseText: response.body, responseHeaders: response.headers
                });
                _this.store.dispatch(requestSuccessAction);
                var /** @type {?} */ end = new Date();
                var /** @type {?} */ duration = end.valueOf() - start.valueOf();
                var /** @type {?} */ requestAsyncAction = {
                    body: parsedResponse,
                    duration: duration,
                    entities: newEntities,
                    meta: meta,
                    status: response.status,
                    text: response.body,
                    transformed: transformed,
                    type: actionTypes.REQUEST_ASYNC,
                };
                if (_this.config && _this.config.mock && _this.config.mock.mode === MockMode.Record) {
                    _this.config.mock.saveMockData(queryKey, [requestSuccessAction, requestAsyncAction]);
                }
                return requestAsyncAction;
            }), retryWhen(function (attempts) {
                var /** @type {?} */ backoff = _this.config && _this.config.backoff || defaultBackoffConfig;
                var /** @type {?} */ retryableStatusCodes = _this.config && _this.config.retryableStatusCodes || defaultRetryableStatusCodes;
                return range(1, backoff.maxAttempts + 1).pipe(zip(attempts, function (i, response) { return ({ response: response, i: i }); }), flatMap(function (_a) {
                    var response = _a.response, i = _a.i;
                    if (i < backoff.maxAttempts && retryableStatusCodes.indexOf(response.status) >= 0) {
                        // Delay retry by min duration the first attempt, up to max duration on the (maxAttempts)th attempt
                        return timer(backoff.minDuration +
                            (i - 1) * (backoff.maxDuration - backoff.minDuration) / backoff.maxAttempts);
                    }
                    else {
                        // Tried maxAttempts, now fail
                        throw response;
                    }
                }));
            }), catchError(function (errResponse, caught) {
                if (errResponse.error instanceof Error) {
                    throw errResponse.error;
                }
                var /** @type {?} */ requestFailureAction = actions.requestFailure({
                    url: url,
                    body: body,
                    status: errResponse.status,
                    responseBody: errResponse.error,
                    meta: meta,
                    queryKey: queryKey,
                    responseText: errResponse.error,
                    responseHeaders: errResponse.headers
                });
                if (_this.config && _this.config.mock && _this.config.mock.mode === MockMode.Record) {
                    _this.config.mock.saveMockData(queryKey, [requestFailureAction]);
                }
                return of(requestFailureAction);
            }));
        }));
        this.mutateAsync = this.actions$.pipe(ofType(ngrxQueryActionTypes.MUTATE_ASYNC), map(function (action) {
            var url = action.url, _a = action.transform, transform = _a === void 0 ? identity : _a, update = action.update, body = action.body, meta = action.meta, optimisticUpdate = action.optimisticUpdate, _b = action.options, options = _b === void 0 ? ({}) : _b;
            invariant(!!url, 'Missing required `url` field in action handler');
            var /** @type {?} */ state = getLatest(_this.store);
            var /** @type {?} */ entities = _this.config && _this.config.entitiesSelector && _this.config.entitiesSelector(state)
                || defaultEntitiesSelector(state);
            var /** @type {?} */ optimisticEntities;
            if (optimisticUpdate) {
                optimisticEntities = optimisticUpdateEntities(optimisticUpdate, entities);
            }
            var /** @type {?} */ queryKey = getQueryKey(action);
            // const start = new Date();
            var _c = options.method, method = _c === void 0 ? httpMethods.POST : _c;
            var /** @type {?} */ request = {
                body: body,
                headers: new HttpHeaders(options.headers),
                method: method,
                observe: /** @type {?} */ ((options.observe || 'response')),
                responseType: options.responseType,
                url: url,
                withCredentials: options.credentials === 'include',
            };
            // Note: only the entities that are included in `optimisticUpdate` will be passed along in the
            // `mutateStart` action as `optimisticEntities`
            _this.store.dispatch(actions.mutateStart({ url: url, body: body, request: request, optimisticEntities: optimisticEntities, queryKey: queryKey, meta: meta }));
            return Object.assign({}, action, { request: request, entities: entities, queryKey: queryKey });
        }), filter(function (action) {
            if (!_this.config || !_this.config.mock || _this.config.mock.mode !== MockMode.Mock) {
                // Not mocking so continue
                return true;
            }
            var url = action.url, body = action.body, meta = action.meta, queryKey = action.queryKey;
            var /** @type {?} */ mockActions = _this.config.mock.getMockData(queryKey);
            if (mockActions) {
                mockActions.forEach(function (mockAction) {
                    _this.store.dispatch(mockAction);
                });
            }
            else {
                _this.store.dispatch(actions.requestFailure({
                    url: url,
                    body: body,
                    status: 500,
                    responseBody: 'No mock data recorded for ' + queryKey,
                    meta: meta,
                    queryKey: queryKey,
                }));
            }
            // Dispatched mock actions so abort network call
            return false;
        }), mergeMap(function (action) {
            var url = action.url, _a = action.transform, transform = _a === void 0 ? identity : _a, update = action.update, body = action.body, meta = action.meta, request = action.request, entities = action.entities, queryKey = action.queryKey;
            return _this.http.request(request.method, url, request).pipe(map(function (response) {
                if (!response.ok) {
                    throw response;
                }
                var /** @type {?} */ status = (response && response.status) || 0;
                var /** @type {?} */ responseBody = (response && response.body) || undefined;
                var /** @type {?} */ responseText = (response && response.body) || undefined;
                var /** @type {?} */ transformed = transform(responseBody, responseText, response);
                var /** @type {?} */ newEntities = updateEntities(update, entities, transformed);
                return actions.mutateSuccess({
                    url: url, body: body, status: status, entities: newEntities, queryKey: queryKey, responseBody: responseBody, responseText: responseText, responseHeaders: response.headers, meta: meta
                });
            }), catchError(function (errResponse) {
                if (errResponse.error instanceof Error) {
                    throw errResponse.error;
                }
                return of(actions.mutateFailure({
                    url: url, body: body, status: errResponse.status, rolledBackEntities: entities, queryKey: queryKey,
                    responseBody: errResponse.error, responseText: errResponse.error, responseHeaders: errResponse.headers, meta: meta
                }));
            }));
        }));
    }
    return NgrxQueryEffects;
}());
export { NgrxQueryEffects };
NgrxQueryEffects.decorators = [
    { type: Injectable },
];
/** @nocollapse */
NgrxQueryEffects.ctorParameters = function () { return [
    { type: Actions, },
    { type: Store, },
    { type: HttpClient, },
    { type: undefined, decorators: [{ type: Inject, args: [NGRX_QUERY_CONFIG,] },] },
]; };
tslib_1.__decorate([
    Effect(),
    tslib_1.__metadata("design:type", Observable)
], NgrxQueryEffects.prototype, "requestAsync", void 0);
tslib_1.__decorate([
    Effect(),
    tslib_1.__metadata("design:type", Observable)
], NgrxQueryEffects.prototype, "mutateAsync", void 0);
function NgrxQueryEffects_tsickle_Closure_declarations() {
    /** @type {!Array<{type: !Function, args: (undefined|!Array<?>)}>} */
    NgrxQueryEffects.decorators;
    /**
     * @nocollapse
     * @type {function(): !Array<(null|{type: ?, decorators: (undefined|!Array<{type: !Function, args: (undefined|!Array<?>)}>)})>}
     */
    NgrxQueryEffects.ctorParameters;
    /** @type {?} */
    NgrxQueryEffects.prototype.requestAsync;
    /** @type {?} */
    NgrxQueryEffects.prototype.mutateAsync;
    /** @type {?} */
    NgrxQueryEffects.prototype.actions$;
    /** @type {?} */
    NgrxQueryEffects.prototype.store;
    /** @type {?} */
    NgrxQueryEffects.prototype.http;
    /** @type {?} */
    NgrxQueryEffects.prototype.config;
}
//# sourceMappingURL=effects.service.js.map
