import { requestAsync, mutateAsync, requestStart, requestFailure, requestSuccess, mutateStart, mutateSuccess, mutateFailure } from 'redux-query/dist/es/actions';
import { InjectionToken, Inject, Injectable, forwardRef, ChangeDetectorRef, Optional, Directive, EventEmitter, Input, Output, NgModule } from '@angular/core';
import { Store } from '@ngrx/store';
import { withoutPath } from 'redux-query/dist/es/reducers/entities';
export { default as entitiesReducer } from 'redux-query/dist/es/reducers/entities';
import { take, filter, catchError, flatMap, map, mergeMap, retryWhen, tap, zip } from 'rxjs/operators';
import { __decorate, __metadata } from 'tslib';
import { HttpClient, HttpHeaders, HttpClientModule } from '@angular/common/http';
import { Actions, Effect, EffectsModule } from '@ngrx/effects';
import { REQUEST_ASYNC } from 'redux-query/dist/es/constants/action-types';
import { GET, POST } from 'redux-query/dist/es/constants/http-methods';
import { getQueryKey } from 'redux-query/dist/es/lib/query-key';
import { Observable, of, range, timer } from 'rxjs';
export { default as queriesReducer } from 'redux-query/dist/es/reducers/queries';
export { queryCount, lastUpdated, status, isPending, isFinished } from 'redux-query/dist/es/selectors/query';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const /** @type {?} */ REQUEST_ASYNC$1 = '[ngrx-query] REQUEST_ASYNC';
const /** @type {?} */ MUTATE_ASYNC = '[ngrx-query] MUTATE_ASYNC';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const /** @type {?} */ requestAsync$1 = (params) => Object.assign({}, requestAsync(params), { type: REQUEST_ASYNC$1 });
const /** @type {?} */ mutateAsync$1 = (params) => Object.assign({}, mutateAsync(params), { type: MUTATE_ASYNC });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const /** @type {?} */ statusCodes = {
    GATEWAY_TIMEOUT: 504,
    REQUEST_TIMEOUT: 408,
    // client took too long
    SERVICE_UNAVAILABLE: 503,
    TOO_MANY_REQUESTS: 429,
    UNKNOWN: 0,
};
const /** @type {?} */ NGRX_QUERY_CONFIG = new InjectionToken('NGRX_QUERY_CONFIG');
const /** @type {?} */ defaultEntitiesSelector = state => state.entities;
const /** @type {?} */ defaultQueriesSelector = state => state.queries;
const /** @type {?} */ defaultBackoffConfig = {
    maxAttempts: 5,
    maxDuration: 5000,
    minDuration: 300,
};
const /** @type {?} */ defaultRetryableStatusCodes = [
    statusCodes.UNKNOWN,
    // normally means a failed connection
    statusCodes.REQUEST_TIMEOUT,
    statusCodes.TOO_MANY_REQUESTS,
    // hopefully backoff stops this getting worse
    statusCodes.SERVICE_UNAVAILABLE,
    statusCodes.GATEWAY_TIMEOUT,
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ConnectService {
    /**
     * @param {?} store
     * @param {?} config
     */
    constructor(store, config) {
        this.store = store;
        this.config = config;
    }
    /**
     * @param {?} config
     * @return {?}
     */
    mutateAsync(config) {
        this.store.dispatch(mutateAsync$1(config));
        return this.select(config.selector);
    }
    /**
     * @param {?} config
     * @return {?}
     */
    requestAsync(config) {
        this.store.dispatch(requestAsync$1(config));
        return this.select(config.selector);
    }
    /**
     * @param {?} selector
     * @return {?}
     */
    select(selector) {
        return this.store.select(selector || this.config && this.config.entitiesSelector || defaultEntitiesSelector);
    }
}
ConnectService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
ConnectService.ctorParameters = () => [
    { type: Store, },
    { type: undefined, decorators: [{ type: Inject, args: [NGRX_QUERY_CONFIG,] },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @abstract
 */
class NqConnectedComponent {
}
/**
 * @param {?} component
 * @param {?=} parentType
 * @return {?}
 */
function provideNqConnectedComponent(component, parentType) {
    return { provide: parentType || NqConnectedComponent, useExisting: forwardRef(() => component) };
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ConnectRequestDirective {
    /**
     * @param {?} connectService
     * @param {?} changeDetector
     * @param {?} host
     */
    constructor(connectService, changeDetector, host) {
        this.connectService = connectService;
        this.changeDetector = changeDetector;
        this.host = host;
        // tslint:disable-next-line:no-input-rename
        this.config = /** @type {?} */ (undefined);
        this.response = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.subscribe(this.config);
        if (this.host) {
            this.host.nqRefresh.subscribe(() => this.forceRequest());
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.unsubscribe();
    }
    /**
     * @param {?} config
     * @return {?}
     */
    subscribe(config) {
        this.unsubscribe();
        this.subscription = this.connectService.requestAsync(config).subscribe(response => {
            if (this.response) {
                if (this.host) {
                    setTimeout(() => {
                        this.host.nqData = response;
                        if (this.host.changeDetector) {
                            this.host.changeDetector.markForCheck();
                        }
                    });
                }
                this.response.emit(response);
            }
        });
    }
    /**
     * @return {?}
     */
    unsubscribe() {
        if (this.subscription) {
            this.subscription.unsubscribe();
        }
    }
    /**
     * @return {?}
     */
    forceRequest() {
        this.subscribe(Object.assign({}, this.config, { force: true }));
    }
}
ConnectRequestDirective.decorators = [
    { type: Directive, args: [{
                exportAs: 'nqConnect',
                selector: '[nqConnect]',
            },] },
];
/** @nocollapse */
ConnectRequestDirective.ctorParameters = () => [
    { type: ConnectService, },
    { type: ChangeDetectorRef, },
    { type: NqConnectedComponent, decorators: [{ type: Optional },] },
];
ConnectRequestDirective.propDecorators = {
    "config": [{ type: Input, args: ['nqConnect',] },],
    "response": [{ type: Output },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @param {?} obj
 * @return {?}
 */
function objectsByIdToArray(obj) {
    if (!obj) {
        return [];
    }
    return Object.keys(obj).map(key => obj[key]);
}
/**
 * @param {?} array
 * @param {?} keyOf
 * @return {?}
 */
function arrayToObjectsById(array, keyOf) {
    return array.reduce((objectsById, item) => {
        objectsById[keyOf(item)] = item;
        return objectsById;
    }, {});
}
/**
 * @template E
 */
class ConnectEntityService {
    /**
     * @param {?} connectService
     */
    constructor(connectService) {
        this.connectService = connectService;
    }
    /**
     * @param {?} config
     * @return {?}
     */
    init(config) {
        this.config = config;
    }
    /**
     * @return {?}
     */
    list() {
        return this.connectService.requestAsync({
            selector: s => objectsByIdToArray(s.entities[this.config.entityTypeName]),
            transform: (response) => ({ [this.config.entityTypeName]: arrayToObjectsById(response, this.config.idSelector) }),
            update: {
                [this.config.entityTypeName]: (prevEntities, entities) => (Object.assign({}, prevEntities, entities)),
            },
            url: this.config.endpoints.list,
        });
    }
    /**
     * @param {?} id
     * @return {?}
     */
    get(id) {
        return this.connectService.requestAsync({
            selector: s => s.entities[this.config.entityTypeName] && s.entities[this.config.entityTypeName][id],
            transform: (response) => ({ [this.config.entityTypeName]: { [id]: response } }),
            update: {
                [this.config.entityTypeName]: (prevEntities, entities) => (Object.assign({}, prevEntities, entities)),
            },
            url: this.config.endpoints.getById(id),
        });
    }
    /**
     * @param {?} entity
     * @return {?}
     */
    create(entity) {
        return this.connectService.mutateAsync({
            body: entity,
            optimisticUpdate: {
                [this.config.entityTypeName]: (prevEntities) => (Object.assign({}, prevEntities, { [this.config.idSelector(entity)]: entity })),
            },
            transform: (response) => ({ [this.config.entityTypeName]: { [this.config.idSelector(response)]: response } }),
            update: {
                [this.config.entityTypeName]: (prevEntities, entities) => (Object.assign({}, prevEntities, entities)),
            },
            url: this.config.endpoints.create,
        });
    }
    /**
     * @param {?} entity
     * @return {?}
     */
    edit(entity) {
        return this.connectService.mutateAsync({
            body: entity,
            optimisticUpdate: {
                [this.config.entityTypeName]: (prevEntities) => (Object.assign({}, prevEntities, { [this.config.idSelector(entity)]: entity })),
            },
            options: {
                method: 'PUT',
            },
            selector: s => s.entities[this.config.entityTypeName][this.config.idSelector(entity)],
            transform: (response) => ({ [this.config.entityTypeName]: { [this.config.idSelector(response)]: response } }),
            update: {
                [this.config.entityTypeName]: (prevEntities, entities) => (Object.assign({}, prevEntities, entities)),
            },
            url: this.config.endpoints.update,
        });
    }
    /**
     * @param {?} id
     * @return {?}
     */
    delete(id) {
        return this.connectService.mutateAsync({
            selector: s => s.entities[this.config.entityTypeName][id],
            optimisticUpdate: {
                [this.config.entityTypeName]: prevEntities => withoutPath(prevEntities, [id]),
            },
            options: {
                method: 'DELETE',
            },
            update: {
                [this.config.entityTypeName]: prevEntities => withoutPath(prevEntities, [id]),
            },
            url: this.config.endpoints.remove(id + ''),
        });
    }
}
ConnectEntityService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
ConnectEntityService.ctorParameters = () => [
    { type: ConnectService, },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @template T
 * @param {?} o$
 * @return {?}
 */
function getLatest(o$) {
    let /** @type {?} */ result;
    o$.pipe(take(1)).subscribe(x => result = x);
    return result;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @param {?} condition
 * @param {?} message
 * @param {?=} context
 * @return {?}
 */
function invariant(condition, message, context) {
    if (!condition) {
        let /** @type {?} */ errorMessage = message;
        if (context) {
            errorMessage = (message.indexOf('%s') !== -1) ?
                message.replace('%s', context) :
                errorMessage = `${message}: ${context}`;
        }
        throw new Error(errorMessage);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/** @enum {number} */
const MockMode = {
    None: 0,
    Record: 1,
    Mock: 2,
};
MockMode[MockMode.None] = "None";
MockMode[MockMode.Record] = "Record";
MockMode[MockMode.Mock] = "Mock";

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const /** @type {?} */ ofType = (actionType) => (source) => source.pipe(filter(action => action.type === actionType));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @param {?} x
 * @param {?=} y
 * @param {?=} z
 * @return {?}
 */
function identity(x, y, z) {
    return x;
}
/**
 * @param {?} optimisticUpdate
 * @param {?} entities
 * @return {?}
 */
function optimisticUpdateEntities(optimisticUpdate, entities) {
    return Object.keys(optimisticUpdate).reduce((accum, key) => {
        if (optimisticUpdate[key]) {
            accum[key] = optimisticUpdate[key](entities[key]);
        }
        else {
            accum[key] = entities[key];
        }
        return accum;
    }, {});
}
/**
 * @param {?} update
 * @param {?} entities
 * @param {?} transformed
 * @return {?}
 */
function updateEntities(update, entities, transformed) {
    // If update not supplied, then no change to entities should be made
    return Object.keys(update || {}).reduce((accum, key) => {
        accum[key] = update[key]((entities || {})[key], (transformed || {})[key]);
        return accum;
    }, {});
}
class NgrxQueryEffects {
    /**
     * @param {?} actions$
     * @param {?} store
     * @param {?} http
     * @param {?} config
     */
    constructor(actions$, store, http, config) {
        this.actions$ = actions$;
        this.store = store;
        this.http = http;
        this.config = config;
        this.requestAsync = this.actions$.pipe(ofType(REQUEST_ASYNC$1), map((action) => {
            const { url, force, retry, update, } = action;
            invariant(!!url, 'Missing required `url` field in action handler');
            invariant(!!update, 'Missing required `update` field in action handler');
            const /** @type {?} */ queryKey = getQueryKey(action);
            const /** @type {?} */ state = getLatest(this.store);
            const /** @type {?} */ queries = this.config && this.config.queriesSelector && this.config.queriesSelector(state)
                || defaultQueriesSelector(state);
            const /** @type {?} */ queriesState = queries[queryKey];
            const { isPending, status } = queriesState || /** @type {?} */ ({});
            const /** @type {?} */ hasSucceeded = status >= 200 && status < 300;
            const /** @type {?} */ attemptRequest = force || !queriesState || (retry !== false && !isPending && !hasSucceeded);
            return { attemptRequest, action };
        }), filter(({ attemptRequest }) => attemptRequest), mergeMap(({ action }) => {
            const { url, body, transform = identity, update, options = /** @type {?} */ ({}), meta, } = action;
            const /** @type {?} */ queryKey = getQueryKey(action);
            const /** @type {?} */ start = new Date();
            const { method = GET } = options;
            const /** @type {?} */ request = {
                body,
                headers: options.headers,
                method,
                observe: /** @type {?} */ ((options.observe || 'response')),
                responseType: options.responseType,
                url,
                withCredentials: options.credentials === 'include',
            };
            return of({}).pipe(tap(() => {
                this.store.dispatch(requestStart({ url, body, request, meta, queryKey }));
            }), filter(() => {
                if (this.config && this.config.mock && this.config.mock.mode === MockMode.Mock) {
                    const /** @type {?} */ mockActions = this.config.mock.getMockData(queryKey);
                    if (mockActions) {
                        mockActions.forEach(mockAction => {
                            this.store.dispatch(mockAction);
                        });
                    }
                    else {
                        this.store.dispatch(requestFailure({
                            url,
                            body,
                            status: 500,
                            responseBody: 'No mock data recorded for ' + queryKey,
                            meta,
                            queryKey,
                        }));
                    }
                    return false;
                }
                return true;
            }), mergeMap(() => {
                return this.http.request(request.method, url, request)
                    .pipe(map(response => {
                    if (!response.ok) {
                        throw response;
                    }
                    return response;
                }));
            }), map(response => {
                const /** @type {?} */ callbackState = getLatest(this.store);
                const /** @type {?} */ entities = this.config && this.config.entitiesSelector && this.config.entitiesSelector(callbackState)
                    || defaultEntitiesSelector(callbackState);
                const /** @type {?} */ parsedResponse = response.body;
                const /** @type {?} */ transformed = transform(parsedResponse, response.body, response);
                const /** @type {?} */ newEntities = updateEntities(update, entities, transformed);
                const /** @type {?} */ requestSuccessAction = requestSuccess({
                    url, body, status: response.status, entities: newEntities, meta, queryKey, responseBody: parsedResponse,
                    responseText: response.body, responseHeaders: response.headers
                });
                this.store.dispatch(requestSuccessAction);
                const /** @type {?} */ end = new Date();
                const /** @type {?} */ duration = end.valueOf() - start.valueOf();
                const /** @type {?} */ requestAsyncAction = {
                    body: parsedResponse,
                    duration,
                    entities: newEntities,
                    meta,
                    status: response.status,
                    text: response.body,
                    transformed,
                    type: REQUEST_ASYNC,
                };
                if (this.config && this.config.mock && this.config.mock.mode === MockMode.Record) {
                    this.config.mock.saveMockData(queryKey, [requestSuccessAction, requestAsyncAction]);
                }
                return requestAsyncAction;
            }), retryWhen(attempts => {
                const /** @type {?} */ backoff = this.config && this.config.backoff || defaultBackoffConfig;
                const /** @type {?} */ retryableStatusCodes = this.config && this.config.retryableStatusCodes || defaultRetryableStatusCodes;
                return range(1, backoff.maxAttempts + 1).pipe(zip(attempts, (i, response) => ({ response, i })), flatMap(({ response, i }) => {
                    if (i < backoff.maxAttempts && retryableStatusCodes.indexOf(response.status) >= 0) {
                        // Delay retry by min duration the first attempt, up to max duration on the (maxAttempts)th attempt
                        return timer(backoff.minDuration +
                            (i - 1) * (backoff.maxDuration - backoff.minDuration) / backoff.maxAttempts);
                    }
                    else {
                        // Tried maxAttempts, now fail
                        throw response;
                    }
                }));
            }), catchError((errResponse, caught) => {
                if (errResponse.error instanceof Error) {
                    throw errResponse.error;
                }
                const /** @type {?} */ requestFailureAction = requestFailure({
                    url,
                    body,
                    status: errResponse.status,
                    responseBody: errResponse.error,
                    meta,
                    queryKey,
                    responseText: errResponse.error,
                    responseHeaders: errResponse.headers
                });
                if (this.config && this.config.mock && this.config.mock.mode === MockMode.Record) {
                    this.config.mock.saveMockData(queryKey, [requestFailureAction]);
                }
                return of(requestFailureAction);
            }));
        }));
        this.mutateAsync = this.actions$.pipe(ofType(MUTATE_ASYNC), map((action) => {
            const { url, transform = identity, update, body, meta, optimisticUpdate, options = /** @type {?} */ ({}), } = action;
            invariant(!!url, 'Missing required `url` field in action handler');
            const /** @type {?} */ state = getLatest(this.store);
            const /** @type {?} */ entities = this.config && this.config.entitiesSelector && this.config.entitiesSelector(state)
                || defaultEntitiesSelector(state);
            let /** @type {?} */ optimisticEntities;
            if (optimisticUpdate) {
                optimisticEntities = optimisticUpdateEntities(optimisticUpdate, entities);
            }
            const /** @type {?} */ queryKey = getQueryKey(action);
            // const start = new Date();
            const { method = POST } = options;
            const /** @type {?} */ request = {
                body,
                headers: new HttpHeaders(options.headers),
                method,
                observe: /** @type {?} */ ((options.observe || 'response')),
                responseType: options.responseType,
                url,
                withCredentials: options.credentials === 'include',
            };
            // Note: only the entities that are included in `optimisticUpdate` will be passed along in the
            // `mutateStart` action as `optimisticEntities`
            this.store.dispatch(mutateStart({ url, body, request, optimisticEntities, queryKey, meta }));
            return Object.assign({}, action, { request, entities, queryKey });
        }), filter((action) => {
            if (!this.config || !this.config.mock || this.config.mock.mode !== MockMode.Mock) {
                // Not mocking so continue
                return true;
            }
            const { url, body, meta, queryKey, } = action;
            const /** @type {?} */ mockActions = this.config.mock.getMockData(queryKey);
            if (mockActions) {
                mockActions.forEach(mockAction => {
                    this.store.dispatch(mockAction);
                });
            }
            else {
                this.store.dispatch(requestFailure({
                    url,
                    body,
                    status: 500,
                    responseBody: 'No mock data recorded for ' + queryKey,
                    meta,
                    queryKey,
                }));
            }
            // Dispatched mock actions so abort network call
            return false;
        }), mergeMap((action) => {
            const { url, transform = identity, update, body, meta, request, entities, queryKey, } = action;
            return this.http.request(request.method, url, request).pipe(map(response => {
                if (!response.ok) {
                    throw response;
                }
                const /** @type {?} */ status = (response && response.status) || 0;
                const /** @type {?} */ responseBody = (response && response.body) || undefined;
                const /** @type {?} */ responseText = (response && response.body) || undefined;
                const /** @type {?} */ transformed = transform(responseBody, responseText, response);
                const /** @type {?} */ newEntities = updateEntities(update, entities, transformed);
                return mutateSuccess({
                    url, body, status, entities: newEntities, queryKey, responseBody, responseText, responseHeaders: response.headers, meta
                });
            }), catchError((errResponse) => {
                if (errResponse.error instanceof Error) {
                    throw errResponse.error;
                }
                return of(mutateFailure({
                    url, body, status: errResponse.status, rolledBackEntities: entities, queryKey,
                    responseBody: errResponse.error, responseText: errResponse.error, responseHeaders: errResponse.headers, meta
                }));
            }));
        }));
    }
}
NgrxQueryEffects.decorators = [
    { type: Injectable },
];
/** @nocollapse */
NgrxQueryEffects.ctorParameters = () => [
    { type: Actions, },
    { type: Store, },
    { type: HttpClient, },
    { type: undefined, decorators: [{ type: Inject, args: [NGRX_QUERY_CONFIG,] },] },
];
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], NgrxQueryEffects.prototype, "requestAsync", void 0);
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], NgrxQueryEffects.prototype, "mutateAsync", void 0);

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class NgrxQueryModule {
    /**
     * @param {?=} config
     * @return {?}
     */
    static forRoot(config) {
        return {
            ngModule: NgrxQueryModule,
            providers: [
                { provide: NGRX_QUERY_CONFIG, useValue: config },
                ConnectService,
                ConnectEntityService,
            ],
        };
    }
}
NgrxQueryModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    ConnectRequestDirective,
                ],
                exports: [
                    ConnectRequestDirective,
                ],
                imports: [
                    EffectsModule.forFeature([NgrxQueryEffects]),
                    HttpClientModule,
                ],
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

export { NgrxQueryModule, NgrxQueryEffects, ConnectService, ConnectEntityService, arrayToObjectsById, objectsByIdToArray, ConnectRequestDirective, NqConnectedComponent, provideNqConnectedComponent, NGRX_QUERY_CONFIG, requestAsync$1 as requestAsync, mutateAsync$1 as mutateAsync, MockMode };
//# sourceMappingURL=ngrx-query.js.map
