import { requestAsync, mutateAsync, requestStart, requestFailure, requestSuccess, mutateStart, mutateSuccess, mutateFailure } from 'redux-query/dist/es/actions';
import { InjectionToken, Inject, Injectable, forwardRef, ChangeDetectorRef, Optional, Directive, EventEmitter, Input, Output, NgModule } from '@angular/core';
import { Store } from '@ngrx/store';
import { withoutPath } from 'redux-query/dist/es/reducers/entities';
export { default as entitiesReducer } from 'redux-query/dist/es/reducers/entities';
import { take, filter, catchError, flatMap, map, mergeMap, retryWhen, tap, zip } from 'rxjs/operators';
import { __decorate, __metadata } from 'tslib';
import { HttpClient, HttpHeaders, HttpClientModule } from '@angular/common/http';
import { Actions, Effect, EffectsModule } from '@ngrx/effects';
import { REQUEST_ASYNC } from 'redux-query/dist/es/constants/action-types';
import { GET, POST } from 'redux-query/dist/es/constants/http-methods';
import { getQueryKey } from 'redux-query/dist/es/lib/query-key';
import { Observable, of, range, timer } from 'rxjs';
export { default as queriesReducer } from 'redux-query/dist/es/reducers/queries';
export { queryCount, lastUpdated, status, isPending, isFinished } from 'redux-query/dist/es/selectors/query';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var /** @type {?} */ REQUEST_ASYNC$1 = '[ngrx-query] REQUEST_ASYNC';
var /** @type {?} */ MUTATE_ASYNC = '[ngrx-query] MUTATE_ASYNC';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var /** @type {?} */ requestAsync$1 = function (params) { return Object.assign({}, requestAsync(params), { type: REQUEST_ASYNC$1 }); };
var /** @type {?} */ mutateAsync$1 = function (params) { return Object.assign({}, mutateAsync(params), { type: MUTATE_ASYNC }); };

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var /** @type {?} */ statusCodes = {
    GATEWAY_TIMEOUT: 504,
    REQUEST_TIMEOUT: 408,
    // client took too long
    SERVICE_UNAVAILABLE: 503,
    TOO_MANY_REQUESTS: 429,
    UNKNOWN: 0,
};
var /** @type {?} */ NGRX_QUERY_CONFIG = new InjectionToken('NGRX_QUERY_CONFIG');
var /** @type {?} */ defaultEntitiesSelector = function (state) { return state.entities; };
var /** @type {?} */ defaultQueriesSelector = function (state) { return state.queries; };
var /** @type {?} */ defaultBackoffConfig = {
    maxAttempts: 5,
    maxDuration: 5000,
    minDuration: 300,
};
var /** @type {?} */ defaultRetryableStatusCodes = [
    statusCodes.UNKNOWN,
    // normally means a failed connection
    statusCodes.REQUEST_TIMEOUT,
    statusCodes.TOO_MANY_REQUESTS,
    // hopefully backoff stops this getting worse
    statusCodes.SERVICE_UNAVAILABLE,
    statusCodes.GATEWAY_TIMEOUT,
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var ConnectService = /** @class */ (function () {
    /**
     * @param {?} store
     * @param {?} config
     */
    function ConnectService(store, config) {
        this.store = store;
        this.config = config;
    }
    /**
     * @param {?} config
     * @return {?}
     */
    ConnectService.prototype.mutateAsync = function (config) {
        this.store.dispatch(mutateAsync$1(config));
        return this.select(config.selector);
    };
    /**
     * @param {?} config
     * @return {?}
     */
    ConnectService.prototype.requestAsync = function (config) {
        this.store.dispatch(requestAsync$1(config));
        return this.select(config.selector);
    };
    /**
     * @param {?} selector
     * @return {?}
     */
    ConnectService.prototype.select = function (selector) {
        return this.store.select(selector || this.config && this.config.entitiesSelector || defaultEntitiesSelector);
    };
    return ConnectService;
}());
ConnectService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
ConnectService.ctorParameters = function () { return [
    { type: Store, },
    { type: undefined, decorators: [{ type: Inject, args: [NGRX_QUERY_CONFIG,] },] },
]; };

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @abstract
 */
var NqConnectedComponent = /** @class */ (function () {
    function NqConnectedComponent() {
    }
    return NqConnectedComponent;
}());
/**
 * @param {?} component
 * @param {?=} parentType
 * @return {?}
 */
function provideNqConnectedComponent(component, parentType) {
    return { provide: parentType || NqConnectedComponent, useExisting: forwardRef(function () { return component; }) };
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var ConnectRequestDirective = /** @class */ (function () {
    /**
     * @param {?} connectService
     * @param {?} changeDetector
     * @param {?} host
     */
    function ConnectRequestDirective(connectService, changeDetector, host) {
        this.connectService = connectService;
        this.changeDetector = changeDetector;
        this.host = host;
        // tslint:disable-next-line:no-input-rename
        this.config = /** @type {?} */ (undefined);
        this.response = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ConnectRequestDirective.prototype.ngOnInit = function () {
        var _this = this;
        this.subscribe(this.config);
        if (this.host) {
            this.host.nqRefresh.subscribe(function () { return _this.forceRequest(); });
        }
    };
    /**
     * @return {?}
     */
    ConnectRequestDirective.prototype.ngOnDestroy = function () {
        this.unsubscribe();
    };
    /**
     * @param {?} config
     * @return {?}
     */
    ConnectRequestDirective.prototype.subscribe = function (config) {
        var _this = this;
        this.unsubscribe();
        this.subscription = this.connectService.requestAsync(config).subscribe(function (response) {
            if (_this.response) {
                if (_this.host) {
                    setTimeout(function () {
                        _this.host.nqData = response;
                        if (_this.host.changeDetector) {
                            _this.host.changeDetector.markForCheck();
                        }
                    });
                }
                _this.response.emit(response);
            }
        });
    };
    /**
     * @return {?}
     */
    ConnectRequestDirective.prototype.unsubscribe = function () {
        if (this.subscription) {
            this.subscription.unsubscribe();
        }
    };
    /**
     * @return {?}
     */
    ConnectRequestDirective.prototype.forceRequest = function () {
        this.subscribe(Object.assign({}, this.config, { force: true }));
    };
    return ConnectRequestDirective;
}());
ConnectRequestDirective.decorators = [
    { type: Directive, args: [{
                exportAs: 'nqConnect',
                selector: '[nqConnect]',
            },] },
];
/** @nocollapse */
ConnectRequestDirective.ctorParameters = function () { return [
    { type: ConnectService, },
    { type: ChangeDetectorRef, },
    { type: NqConnectedComponent, decorators: [{ type: Optional },] },
]; };
ConnectRequestDirective.propDecorators = {
    "config": [{ type: Input, args: ['nqConnect',] },],
    "response": [{ type: Output },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @param {?} obj
 * @return {?}
 */
function objectsByIdToArray(obj) {
    if (!obj) {
        return [];
    }
    return Object.keys(obj).map(function (key) { return obj[key]; });
}
/**
 * @param {?} array
 * @param {?} keyOf
 * @return {?}
 */
function arrayToObjectsById(array, keyOf) {
    return array.reduce(function (objectsById, item) {
        objectsById[keyOf(item)] = item;
        return objectsById;
    }, {});
}
/**
 * @template E
 */
var ConnectEntityService = /** @class */ (function () {
    /**
     * @param {?} connectService
     */
    function ConnectEntityService(connectService) {
        this.connectService = connectService;
    }
    /**
     * @param {?} config
     * @return {?}
     */
    ConnectEntityService.prototype.init = function (config) {
        this.config = config;
    };
    /**
     * @return {?}
     */
    ConnectEntityService.prototype.list = function () {
        var _this = this;
        return this.connectService.requestAsync({
            selector: function (s) { return objectsByIdToArray(s.entities[_this.config.entityTypeName]); },
            transform: function (response) {
                return (_a = {}, _a[_this.config.entityTypeName] = arrayToObjectsById(response, _this.config.idSelector), _a);
                var _a;
            },
            update: (_a = {},
                _a[this.config.entityTypeName] = function (prevEntities, entities) { return (Object.assign({}, prevEntities, entities)); },
                _a),
            url: this.config.endpoints.list,
        });
        var _a;
    };
    /**
     * @param {?} id
     * @return {?}
     */
    ConnectEntityService.prototype.get = function (id) {
        var _this = this;
        return this.connectService.requestAsync({
            selector: function (s) { return s.entities[_this.config.entityTypeName] && s.entities[_this.config.entityTypeName][id]; },
            transform: function (response) {
                return (_a = {}, _a[_this.config.entityTypeName] = (_b = {}, _b[id] = response, _b), _a);
                var _a, _b;
            },
            update: (_a = {},
                _a[this.config.entityTypeName] = function (prevEntities, entities) { return (Object.assign({}, prevEntities, entities)); },
                _a),
            url: this.config.endpoints.getById(id),
        });
        var _a;
    };
    /**
     * @param {?} entity
     * @return {?}
     */
    ConnectEntityService.prototype.create = function (entity) {
        var _this = this;
        return this.connectService.mutateAsync({
            body: entity,
            optimisticUpdate: (_a = {},
                _a[this.config.entityTypeName] = function (prevEntities) {
                    return (Object.assign({}, prevEntities, (_a = {}, _a[_this.config.idSelector(entity)] = entity, _a)));
                    var _a;
                },
                _a),
            transform: function (response) {
                return (_a = {}, _a[_this.config.entityTypeName] = (_b = {}, _b[_this.config.idSelector(response)] = response, _b), _a);
                var _a, _b;
            },
            update: (_b = {},
                _b[this.config.entityTypeName] = function (prevEntities, entities) { return (Object.assign({}, prevEntities, entities)); },
                _b),
            url: this.config.endpoints.create,
        });
        var _a, _b;
    };
    /**
     * @param {?} entity
     * @return {?}
     */
    ConnectEntityService.prototype.edit = function (entity) {
        var _this = this;
        return this.connectService.mutateAsync({
            body: entity,
            optimisticUpdate: (_a = {},
                _a[this.config.entityTypeName] = function (prevEntities) {
                    return (Object.assign({}, prevEntities, (_a = {}, _a[_this.config.idSelector(entity)] = entity, _a)));
                    var _a;
                },
                _a),
            options: {
                method: 'PUT',
            },
            selector: function (s) { return s.entities[_this.config.entityTypeName][_this.config.idSelector(entity)]; },
            transform: function (response) {
                return (_a = {}, _a[_this.config.entityTypeName] = (_b = {}, _b[_this.config.idSelector(response)] = response, _b), _a);
                var _a, _b;
            },
            update: (_b = {},
                _b[this.config.entityTypeName] = function (prevEntities, entities) { return (Object.assign({}, prevEntities, entities)); },
                _b),
            url: this.config.endpoints.update,
        });
        var _a, _b;
    };
    /**
     * @param {?} id
     * @return {?}
     */
    ConnectEntityService.prototype.delete = function (id) {
        var _this = this;
        return this.connectService.mutateAsync({
            selector: function (s) { return s.entities[_this.config.entityTypeName][id]; },
            optimisticUpdate: (_a = {},
                _a[this.config.entityTypeName] = function (prevEntities) { return withoutPath(prevEntities, [id]); },
                _a),
            options: {
                method: 'DELETE',
            },
            update: (_b = {},
                _b[this.config.entityTypeName] = function (prevEntities) { return withoutPath(prevEntities, [id]); },
                _b),
            url: this.config.endpoints.remove(id + ''),
        });
        var _a, _b;
    };
    return ConnectEntityService;
}());
ConnectEntityService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
ConnectEntityService.ctorParameters = function () { return [
    { type: ConnectService, },
]; };

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @template T
 * @param {?} o$
 * @return {?}
 */
function getLatest(o$) {
    var /** @type {?} */ result;
    o$.pipe(take(1)).subscribe(function (x) { return result = x; });
    return result;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @param {?} condition
 * @param {?} message
 * @param {?=} context
 * @return {?}
 */
function invariant(condition, message, context) {
    if (!condition) {
        var /** @type {?} */ errorMessage = message;
        if (context) {
            errorMessage = (message.indexOf('%s') !== -1) ?
                message.replace('%s', context) :
                errorMessage = message + ": " + context;
        }
        throw new Error(errorMessage);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/** @enum {number} */
var MockMode = {
    None: 0,
    Record: 1,
    Mock: 2,
};
MockMode[MockMode.None] = "None";
MockMode[MockMode.Record] = "Record";
MockMode[MockMode.Mock] = "Mock";

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var /** @type {?} */ ofType = function (actionType) { return function (source) { return source.pipe(filter(function (action) { return action.type === actionType; })); }; };

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @param {?} x
 * @param {?=} y
 * @param {?=} z
 * @return {?}
 */
function identity(x, y, z) {
    return x;
}
/**
 * @param {?} optimisticUpdate
 * @param {?} entities
 * @return {?}
 */
function optimisticUpdateEntities(optimisticUpdate, entities) {
    return Object.keys(optimisticUpdate).reduce(function (accum, key) {
        if (optimisticUpdate[key]) {
            accum[key] = optimisticUpdate[key](entities[key]);
        }
        else {
            accum[key] = entities[key];
        }
        return accum;
    }, {});
}
/**
 * @param {?} update
 * @param {?} entities
 * @param {?} transformed
 * @return {?}
 */
function updateEntities(update, entities, transformed) {
    // If update not supplied, then no change to entities should be made
    return Object.keys(update || {}).reduce(function (accum, key) {
        accum[key] = update[key]((entities || {})[key], (transformed || {})[key]);
        return accum;
    }, {});
}
var NgrxQueryEffects = /** @class */ (function () {
    /**
     * @param {?} actions$
     * @param {?} store
     * @param {?} http
     * @param {?} config
     */
    function NgrxQueryEffects(actions$, store, http, config) {
        var _this = this;
        this.actions$ = actions$;
        this.store = store;
        this.http = http;
        this.config = config;
        this.requestAsync = this.actions$.pipe(ofType(REQUEST_ASYNC$1), map(function (action) {
            var url = action.url, force = action.force, retry = action.retry, update = action.update;
            invariant(!!url, 'Missing required `url` field in action handler');
            invariant(!!update, 'Missing required `update` field in action handler');
            var /** @type {?} */ queryKey = getQueryKey(action);
            var /** @type {?} */ state = getLatest(_this.store);
            var /** @type {?} */ queries = _this.config && _this.config.queriesSelector && _this.config.queriesSelector(state)
                || defaultQueriesSelector(state);
            var /** @type {?} */ queriesState = queries[queryKey];
            var _a = queriesState || /** @type {?} */ ({}), isPending = _a.isPending, status = _a.status;
            var /** @type {?} */ hasSucceeded = status >= 200 && status < 300;
            var /** @type {?} */ attemptRequest = force || !queriesState || (retry !== false && !isPending && !hasSucceeded);
            return { attemptRequest: attemptRequest, action: action };
        }), filter(function (_a) {
            var attemptRequest = _a.attemptRequest;
            return attemptRequest;
        }), mergeMap(function (_a) {
            var action = _a.action;
            var url = action.url, body = action.body, _b = action.transform, transform = _b === void 0 ? identity : _b, update = action.update, _c = action.options, options = _c === void 0 ? ({}) : _c, meta = action.meta;
            var /** @type {?} */ queryKey = getQueryKey(action);
            var /** @type {?} */ start = new Date();
            var _d = options.method, method = _d === void 0 ? GET : _d;
            var /** @type {?} */ request = {
                body: body,
                headers: options.headers,
                method: method,
                observe: /** @type {?} */ ((options.observe || 'response')),
                responseType: options.responseType,
                url: url,
                withCredentials: options.credentials === 'include',
            };
            return of({}).pipe(tap(function () {
                _this.store.dispatch(requestStart({ url: url, body: body, request: request, meta: meta, queryKey: queryKey }));
            }), filter(function () {
                if (_this.config && _this.config.mock && _this.config.mock.mode === MockMode.Mock) {
                    var /** @type {?} */ mockActions = _this.config.mock.getMockData(queryKey);
                    if (mockActions) {
                        mockActions.forEach(function (mockAction) {
                            _this.store.dispatch(mockAction);
                        });
                    }
                    else {
                        _this.store.dispatch(requestFailure({
                            url: url,
                            body: body,
                            status: 500,
                            responseBody: 'No mock data recorded for ' + queryKey,
                            meta: meta,
                            queryKey: queryKey,
                        }));
                    }
                    return false;
                }
                return true;
            }), mergeMap(function () {
                return _this.http.request(request.method, url, request)
                    .pipe(map(function (response) {
                    if (!response.ok) {
                        throw response;
                    }
                    return response;
                }));
            }), map(function (response) {
                var /** @type {?} */ callbackState = getLatest(_this.store);
                var /** @type {?} */ entities = _this.config && _this.config.entitiesSelector && _this.config.entitiesSelector(callbackState)
                    || defaultEntitiesSelector(callbackState);
                var /** @type {?} */ parsedResponse = response.body;
                var /** @type {?} */ transformed = transform(parsedResponse, response.body, response);
                var /** @type {?} */ newEntities = updateEntities(update, entities, transformed);
                var /** @type {?} */ requestSuccessAction = requestSuccess({
                    url: url, body: body, status: response.status, entities: newEntities, meta: meta, queryKey: queryKey, responseBody: parsedResponse,
                    responseText: response.body, responseHeaders: response.headers
                });
                _this.store.dispatch(requestSuccessAction);
                var /** @type {?} */ end = new Date();
                var /** @type {?} */ duration = end.valueOf() - start.valueOf();
                var /** @type {?} */ requestAsyncAction = {
                    body: parsedResponse,
                    duration: duration,
                    entities: newEntities,
                    meta: meta,
                    status: response.status,
                    text: response.body,
                    transformed: transformed,
                    type: REQUEST_ASYNC,
                };
                if (_this.config && _this.config.mock && _this.config.mock.mode === MockMode.Record) {
                    _this.config.mock.saveMockData(queryKey, [requestSuccessAction, requestAsyncAction]);
                }
                return requestAsyncAction;
            }), retryWhen(function (attempts) {
                var /** @type {?} */ backoff = _this.config && _this.config.backoff || defaultBackoffConfig;
                var /** @type {?} */ retryableStatusCodes = _this.config && _this.config.retryableStatusCodes || defaultRetryableStatusCodes;
                return range(1, backoff.maxAttempts + 1).pipe(zip(attempts, function (i, response) { return ({ response: response, i: i }); }), flatMap(function (_a) {
                    var response = _a.response, i = _a.i;
                    if (i < backoff.maxAttempts && retryableStatusCodes.indexOf(response.status) >= 0) {
                        // Delay retry by min duration the first attempt, up to max duration on the (maxAttempts)th attempt
                        return timer(backoff.minDuration +
                            (i - 1) * (backoff.maxDuration - backoff.minDuration) / backoff.maxAttempts);
                    }
                    else {
                        // Tried maxAttempts, now fail
                        throw response;
                    }
                }));
            }), catchError(function (errResponse, caught) {
                if (errResponse.error instanceof Error) {
                    throw errResponse.error;
                }
                var /** @type {?} */ requestFailureAction = requestFailure({
                    url: url,
                    body: body,
                    status: errResponse.status,
                    responseBody: errResponse.error,
                    meta: meta,
                    queryKey: queryKey,
                    responseText: errResponse.error,
                    responseHeaders: errResponse.headers
                });
                if (_this.config && _this.config.mock && _this.config.mock.mode === MockMode.Record) {
                    _this.config.mock.saveMockData(queryKey, [requestFailureAction]);
                }
                return of(requestFailureAction);
            }));
        }));
        this.mutateAsync = this.actions$.pipe(ofType(MUTATE_ASYNC), map(function (action) {
            var url = action.url, _a = action.transform, update = action.update, body = action.body, meta = action.meta, optimisticUpdate = action.optimisticUpdate, _b = action.options, options = _b === void 0 ? ({}) : _b;
            invariant(!!url, 'Missing required `url` field in action handler');
            var /** @type {?} */ state = getLatest(_this.store);
            var /** @type {?} */ entities = _this.config && _this.config.entitiesSelector && _this.config.entitiesSelector(state)
                || defaultEntitiesSelector(state);
            var /** @type {?} */ optimisticEntities;
            if (optimisticUpdate) {
                optimisticEntities = optimisticUpdateEntities(optimisticUpdate, entities);
            }
            var /** @type {?} */ queryKey = getQueryKey(action);
            // const start = new Date();
            var _c = options.method, method = _c === void 0 ? POST : _c;
            var /** @type {?} */ request = {
                body: body,
                headers: new HttpHeaders(options.headers),
                method: method,
                observe: /** @type {?} */ ((options.observe || 'response')),
                responseType: options.responseType,
                url: url,
                withCredentials: options.credentials === 'include',
            };
            // Note: only the entities that are included in `optimisticUpdate` will be passed along in the
            // `mutateStart` action as `optimisticEntities`
            _this.store.dispatch(mutateStart({ url: url, body: body, request: request, optimisticEntities: optimisticEntities, queryKey: queryKey, meta: meta }));
            return Object.assign({}, action, { request: request, entities: entities, queryKey: queryKey });
        }), filter(function (action) {
            if (!_this.config || !_this.config.mock || _this.config.mock.mode !== MockMode.Mock) {
                // Not mocking so continue
                return true;
            }
            var url = action.url, body = action.body, meta = action.meta, queryKey = action.queryKey;
            var /** @type {?} */ mockActions = _this.config.mock.getMockData(queryKey);
            if (mockActions) {
                mockActions.forEach(function (mockAction) {
                    _this.store.dispatch(mockAction);
                });
            }
            else {
                _this.store.dispatch(requestFailure({
                    url: url,
                    body: body,
                    status: 500,
                    responseBody: 'No mock data recorded for ' + queryKey,
                    meta: meta,
                    queryKey: queryKey,
                }));
            }
            // Dispatched mock actions so abort network call
            return false;
        }), mergeMap(function (action) {
            var url = action.url, _a = action.transform, transform = _a === void 0 ? identity : _a, update = action.update, body = action.body, meta = action.meta, request = action.request, entities = action.entities, queryKey = action.queryKey;
            return _this.http.request(request.method, url, request).pipe(map(function (response) {
                if (!response.ok) {
                    throw response;
                }
                var /** @type {?} */ status = (response && response.status) || 0;
                var /** @type {?} */ responseBody = (response && response.body) || undefined;
                var /** @type {?} */ responseText = (response && response.body) || undefined;
                var /** @type {?} */ transformed = transform(responseBody, responseText, response);
                var /** @type {?} */ newEntities = updateEntities(update, entities, transformed);
                return mutateSuccess({
                    url: url, body: body, status: status, entities: newEntities, queryKey: queryKey, responseBody: responseBody, responseText: responseText, responseHeaders: response.headers, meta: meta
                });
            }), catchError(function (errResponse) {
                if (errResponse.error instanceof Error) {
                    throw errResponse.error;
                }
                return of(mutateFailure({
                    url: url, body: body, status: errResponse.status, rolledBackEntities: entities, queryKey: queryKey,
                    responseBody: errResponse.error, responseText: errResponse.error, responseHeaders: errResponse.headers, meta: meta
                }));
            }));
        }));
    }
    return NgrxQueryEffects;
}());
NgrxQueryEffects.decorators = [
    { type: Injectable },
];
/** @nocollapse */
NgrxQueryEffects.ctorParameters = function () { return [
    { type: Actions, },
    { type: Store, },
    { type: HttpClient, },
    { type: undefined, decorators: [{ type: Inject, args: [NGRX_QUERY_CONFIG,] },] },
]; };
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], NgrxQueryEffects.prototype, "requestAsync", void 0);
__decorate([
    Effect(),
    __metadata("design:type", Observable)
], NgrxQueryEffects.prototype, "mutateAsync", void 0);

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var NgrxQueryModule = /** @class */ (function () {
    function NgrxQueryModule() {
    }
    /**
     * @param {?=} config
     * @return {?}
     */
    NgrxQueryModule.forRoot = function (config) {
        return {
            ngModule: NgrxQueryModule,
            providers: [
                { provide: NGRX_QUERY_CONFIG, useValue: config },
                ConnectService,
                ConnectEntityService,
            ],
        };
    };
    return NgrxQueryModule;
}());
NgrxQueryModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    ConnectRequestDirective,
                ],
                exports: [
                    ConnectRequestDirective,
                ],
                imports: [
                    EffectsModule.forFeature([NgrxQueryEffects]),
                    HttpClientModule,
                ],
            },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

export { NgrxQueryModule, NgrxQueryEffects, ConnectService, ConnectEntityService, arrayToObjectsById, objectsByIdToArray, ConnectRequestDirective, NqConnectedComponent, provideNqConnectedComponent, NGRX_QUERY_CONFIG, requestAsync$1 as requestAsync, mutateAsync$1 as mutateAsync, MockMode };
//# sourceMappingURL=ngrx-query.js.map
